@@ -12,7 +12,7 @@
 #include <stdlib.h>
 #include <time.h>
 #include "copyright.h"
-#include "config.h"
+#include "aconfig.h"
 #include "numlib.h"
 #include "imdi.h"
 #include "refi.h"
