@@ -29,7 +29,7 @@
 #include <time.h>
 #include <string.h>
 #include "copyright.h"
-#include "config.h"
+#include "aconfig.h"
 #include "cgats.h"
 #include "icc.h"
 #include <stdarg.h>
