@@ -47,6 +47,8 @@
 
 #define XICC_USE_HK 1	/* [Set] Set to 1 to use Helmholtz-Kohlraush in all CAM conversions */
 #define XICC_NOCAMCL 0	/* [Unset] Set to 1 to disable clipping out of CAM gamut on XYZ to Jab */
+#define XICC_NEUTRAL_CMYK_BLACK		/* Else use K direction black */
+#define XICC_BLACK_POINT_TOLL 1.0		/* Tollerance of CMYK black point location */ 
 
 /* ------------------------------------------------------------------------------ */
 
@@ -121,7 +123,7 @@ typedef struct {
 } icxInkCurve;
 
 /* Default black generation smoothing value */
-#define ICXINKDEFSMTH 0.20
+#define ICXINKDEFSMTH 0.09	/* Transition window of +/- ICXINKDEFSMTH */
 #define ICXINKDEFSKEW 2.0	/* Curve shape skew (matches typical device behaviour) */
 
 /* Structure to convey inking details */
@@ -232,9 +234,9 @@ struct _xicc {
 #define ICX_SET_WHITE       0x0001		/* find, set and make relative to the white point */
 #define ICX_SET_BLACK       0x0002		/* find and set the black point */
 #define ICX_WRITE_WBL       0x0004		/* Matrix: write White, Black & Luminance tags */
-#define ICX_NO_IN_SHP_LUTS  0x0040		/* If LuLut: Don't create input (Device) shaper curves. */
-#define ICX_NO_IN_POS_LUTS  0x0080		/* If LuLut: Don't create input (Device) poistion curves. */
-#define ICX_NO_OUT_LUTS     0x0100		/* If LuLut: Don't create output (PCS) curves. */
+#define ICX_NO_IN_SHP_LUTS  0x0040		/* Lut/Mtx: Don't create input (Device) shaper curves. */
+#define ICX_NO_IN_POS_LUTS  0x0080		/* LuLut: Don't create input (Device) postion curves. */
+#define ICX_NO_OUT_LUTS     0x0100		/* LuLut: Don't create output (PCS) curves. */
 //#define ICX_2PASSSMTH       0x0400		/* If LuLut: Use Gaussian smoothing */
 //#define ICX_EXTRA_FIT       0x0800		/* If LuLut: Counteract scat data point errors. */
 /* And  ICX_VERBOSE         0x8000 */	/* Turn on verboseness during creation */
